#!/usr/bin/python

import sys,os,time,re

#Check if the script has super user permissions
if os.getuid() != 0:
    print "%s: This script requires super user permissions." % os.path.basename(sys.argv[0])
    sys.exit(1)

try:
    from scripts.RelengErrUtil import RelengErr, make_str
    from scripts import CommandLine
    from scripts.installer_util import CommonUtils, InstallerUtil
except Exception,e:
    print e
    print "Unable to access installer related scripts, Please verify if all the files in the installer are available as per the readme"
    sys.exit()


__doc__     = """TeamForge installer."""

class Setup(InstallerUtil):

    commandline_opts = [
                 CommandLine.Option('-r', '--runtime', key='runtime',
                       descr='Create the runtime instance using the site configuration file'),
                 CommandLine.Option('-f', '--siteoptionsfile', key='siteoptionsfile',
                       type='str',argname='siteoptionsfile',
                       descr='Path to the site configuration file (default: /opt/collabnet/teamforge/etc/site-options.conf)'),
                 CommandLine.Option('-I', '--initscript', key='initscript',
                       descr='Start application services on reboot'),
                 CommandLine.Option('-V', '--verbose', key='verbose',
                       descr='Show all output in noninteractive mode'),
                 CommandLine.Option('-n', '--noninteractive', key='non_interactive',
                       descr='Used to run the installer in non-interactive mode'),
                 CommandLine.Option('-y', '--yes', key='yes',
                       descr='Used to run the installer in non-interactive mode'),                 
                 CommandLine.Option('-b', '--bootstrap', key='bootstrap',
                       descr='Bootstrap the existing site. This will remove the existing data and recreate new schema for the application'),
                 CommandLine.Option('-s', '--skip-verification', key='skip_verification',
                       descr='Used to skip running verification snippets'),

                       ]


    commandline_args = []

    def __init__(self, argv):

        CommonUtils.__init__(self, argv[0], __doc__, 
                            commandline_opts = self.commandline_opts, 
                            commandline_args = self.commandline_args,
                            argv = argv)
    
        self.yes = False
        self.database_type = 'postgresql'
        self.bootstrap = False
        self.skip_verification = False

        # Default installation path
        self.installation_path = '/opt/collabnet/teamforge'

        if self.cmd.get_value('siteoptionsfile'):
           self.siteoptionsfile = self.cmd.get_value('siteoptionsfile')
        else:
           self.siteoptionsfile = os.path.join(self.installation_path,'etc','site-options.conf')

        #Throw error is site-options.conf file is missing
        if not(os.path.isfile(self.siteoptionsfile)):
           raise RelengErr(741)

        self.tmp_path   = os.path.join(os.path.dirname(__file__), 'CNlogs')
        self.logs_path  = os.path.join(self.tmp_path, "logs")

        if self.cmd.get_value('bootstrap'):
           self.bootstrap = True

        if self.cmd.get_value('yes') or self.cmd.get_value('non_interactive'):
           self.yes = True

        if self.cmd.get_value('skip_verification'):
           self.skip_verification = True
 
        if os.path.isfile(self.siteoptionsfile):
           self.database_type = self.get_siteoptions(self.siteoptionsfile, 'DATABASE_TYPE')
           
        self.out("\n\033[0m\033[93mThis command has been deprecated. Please consider using '/opt/collabnet/teamforge/bin/teamforge provision' instead.\n\033[0m")
        time.sleep(2)

        if not(self.yes):
            if (re.match("postgresql", self.database_type)):
                msg = """

You have chosen to configure TeamForge in dedicated mode. The installer will automatically configure parts of your system.

1. PostgreSQL will be configured to make it work with TeamForge.
2. The httpd.conf file will be regenerated to support TeamForge."""
            else:
                msg = """
You have chosen to configure TeamForge in dedicated mode. The installer will automatically configure parts of your system.

* The httpd.conf file will be regenerated to support TeamForge."""

            self.out(msg)

    def summary(self, bootstrap=False):
        self.out("##########################################################################################################################")
        self.out("                                         Starting TeamForge Installer")
        self.out("##########################################################################################################################")
        self.out("\nYou have configured the installer. It will now:\n")
        self.out( "- Create the runtime environment.\n")
        if bootstrap:
            self.out( "- Create the initial data for (bootstrap) the site.\n")
        time.sleep(2)


    def work(self):
        try:
            ret = \
                self.createRuntime(self.installation_path,self.siteoptionsfile,
                   self.skip_verification, self.yes)
            if ret != 0:
                if ret == 2:
                    raise RelengErr(make_str(746))
                else:
                    sys.exit(1)
        except:
            sys.exit(1)

        if self.bootstrap:
            ret = self.BootstrapData(self.installation_path, {'SRC_DIR': os.path.dirname(__file__)}, self.yes)
            if ret != 0:
               raise RelengErr(make_str(735, ret))


    def get_siteoptions(self, siteoptionsfile, token, exitonerror=True):
        value = ''
        if os.path.isfile(siteoptionsfile):
           cmd = "grep ^%s '%s' | cut -f2 -d '='" %(token, siteoptionsfile)
           ret = self.execute_cmd(cmd)
           #TODO: Fewthings we need to incorporate
           #1. This function should be used for reading
           #   site-options token. I see that
           #   validateSiteOptions using similar logic.
           #2. This function should handle the case if
           #   grep returns multiple lines.
           #3. This function should handle commented tokens.
           if not ret[0]:
              value = ret[1]
           else:
              if exitonerror:
                  raise RelengErr(make_str(915, ret[1]))
              else:
                  return None
        else:
           if exitonerror:
               raise RelengErr(make_str(700,siteoptionsfile))
           else:
               return None
        return value


    def run(self):
        self.summary(self.bootstrap)
        self.work()

if __name__ == '__main__':
     setup = Setup(sys.argv)
     setup.run()
